/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Dict
 *
 * The Initial Developer of the Original Code is David Costanzo.
 * Portions created by the Initial Developer are Copyright (C) 2004 - 2005
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

// The preferences that we handle
const PREFNAME_DEFAULTSERVERNAME = "extensions.dict.defaultservername";
const PREFNAME_DEFAULTSERVERPORT = "extensions.dict.defaultserverport";
const PREFNAME_KNOWNSERVERS      = "extensions.dict.knownservers.";

function DictServer (name, port, description) {
    this.name        = name;
    this.port        = port;
    this.description = description;
}

// The list of valid DICT servers (generated by mkdictservertable.pl)
var DEFAULT_DICT_SERVERS = [
    new DictServer(
        'dict.arabeyes.org',
        '2628',
        'English-Arabic'),
    new DictServer(
        'dict.hewgill.com',
        '2628',
        'Formatted entry pages from the Wiktionary project. Offers four databases with varying levels of detail, including a raw wikitext view.'),
    new DictServer(
        'dict.org',
        '2628',
        'The standard dictd server, contains several databases related to English language and the FreeDict databases.'),
    new DictServer(
        'dict.poboxes.info',
        '2628',
        'All standard English databases from dict.org'),
    new DictServer(
        'dict.saugus.net',
        '2628',
        'The contents of the Saugus.net computer knowledgebase, including a section on computer terms, filename extensions, and free software titles.'),
    new DictServer(
        'dict.seesslen.net',
        '2628',
        'Fd-deu-eng, fd-eng-deu, fd-eng-rus'),
    new DictServer(
        'dict.tu-chemnitz.de',
        '2628',
        'Standard databases'),
    new DictServer(
        'dict.tugraz.at',
        '2628',
        'Standard and freedict databases'),
    new DictServer(
        'dict.uni-leipzig.de',
        '2628',
        'Official FreeDict server, most current, partly unreleased FreeDict dictionaries.'),
    new DictServer(
        'dictd.xdsl.by',
        '2628',
        'Diverse databases, many Slavic/Russian dictionaries (this was mova.org previously)'),
    new DictServer(
        'dictionary.bishopston.net',
        '2628',
        'IPv6 enabled standard dict server with standard databases (web1913, wn, gazetteer, jargon, foldoc, elements, easton, hitchcock, vera, devils, world95); now also supports direct lookups of the form http://[query].dictionary.bishopston.net'),
    new DictServer(
        'indica-et-buddhica.org',
        '2628',
        'Indica-et-buddhica.org: Sanskrit-Tibetan Buddhist Terminology, Monier-Williams Sanskrit-English Dictionary'),
    new DictServer(
        'la-sorciere.de',
        '2628',
        'Debian terminology dictionaries'),
    new DictServer(
        'linicks.net',
        '2628',
        'Shakespearean Words and meanings. This dict server is primarily used by crossword addicts to find words Chambers English Dictionary defines as \'Shakespeare\' words.'),
    new DictServer(
        'lividict.org',
        '2628',
        'Livid\'s Lividict, accelerated! You can look up the whole world here.'),
    new DictServer(
        'localhost',
        '2628',
        'The DICT server that is running on your local machine.'),
    new DictServer(
        'nihongobenkyo.org',
        '2628',
        'A server focusing on free Japanese dictionaries'),
    new DictServer(
        'test.dict.org',
        '2628',
        'Same databases as dict.org, but available for more agressive client testing (standard databases, Freedict databases).'),
    new DictServer(
        'www.lojban.org',
        '2628',
        'To and from lojban and a number of other languages.'),
];


// Get the default server name, or use a default value
// Does not throw an exception.
function dictGetDefaultServerName() {

    var pref = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefBranch);

    try {
        return pref.getCharPref(PREFNAME_DEFAULTSERVERNAME);
    }
    catch (e) {
    }

    // something is wrong with the preferences,
    // return a default value
    return 'dict.org';
}

// Get the default server port, or use a default value
// Does not throw an exception.
function dictGetDefaultServerPort() {

    var pref = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefBranch);

    try {
         return pref.getCharPref(PREFNAME_DEFAULTSERVERPORT);
     }
     catch (e) {
     }

    try {
        // maybe it's stored as an int
        return pref.getIntPref(PREFNAME_DEFAULTSERVERPORT);
    }
    catch (e) {
    }

    // something is wrong with the preferences,
    // return a default value
    return '2628';
}

// Set the server name preference.  Throw an exception if it cannot be set
function dictSetDefaultServerName(servername) {

    var pref = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefBranch);
    
    pref.setCharPref(PREFNAME_DEFAULTSERVERNAME, servername);
}

// Set a server port preference.  Throw an exception if it cannot be set
function dictSetDefaultServerPort(serverport) {

    var pref = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefBranch);
    
    try {
        pref.setCharPref(PREFNAME_DEFAULTSERVERPORT, serverport);
    }
    catch (e) {
    }

    // maybe it's stored as an int
    pref.setIntPref(PREFNAME_DEFAULTSERVERPORT, serverport);
}


// Gets the name of the known server at index i.
function dictGetKnownServerInfo(index, part) {

    var pref = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefBranch);

    var prefName = PREFNAME_KNOWNSERVERS + index + "." + part;
    return pref.getCharPref(prefName);
}

// Gets the name of the known server at index.
function dictGetKnownServerName(index) {
    return dictGetKnownServerInfo(index, "name");
}

// Gets the port of the known server at index.
function dictGetKnownServerPort(index) {
    return dictGetKnownServerInfo(index, "port");
}

// Gets the description of the known server at index.
function dictGetKnownServerDescription(index) {
    return dictGetKnownServerInfo(index, "description");
}

// Sets the name of the known server at index.
function dictSetKnownServerInfo(index, part, info) {

    var pref = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefBranch);

    var prefName = PREFNAME_KNOWNSERVERS + index + "." + part;
    pref.setCharPref(prefName, info);
}

// Sets the name of the known server at index.
function dictSetKnownServerName(index, name) {
    return dictSetKnownServerInfo(index, "name", name);
}

// Sets the port of the known server at index.
function dictSetKnownServerPort(index, port) {
    return dictSetKnownServerInfo(index, "port", port);
}

// Sets the description of the known server at index.
function dictSetKnownServerDescription(index, description) {
    return dictSetKnownServerInfo(index, "description", description);
}

// handler for when a new DICT server is selected in the list box
function newDictServerSelected() {

    var descriptionElement = document.getElementById("description"); 

    // remove any existing description
    while (descriptionElement.firstChild) {
        descriptionElement.removeChild(descriptionElement.firstChild);
    }

    var listbox = document.getElementById("dictd-servers");

    var serverID = listbox.selectedIndex;

    // Update the description text to match the description of the
    // newly selected server.  Also update the hidden preference elements.
    var descriptionTextNode;

    try {
        var serverName = dictGetKnownServerName(serverID);
        var serverPort = dictGetKnownServerPort(serverID);
        var serverDescription = dictGetKnownServerDescription(serverID);

        // update the preferences in our hidden preference elements
        var defaultServerName = document.getElementById('defaultservername');
        defaultServerName.setAttribute('value', serverName);

        var defaultServerPort = document.getElementById('defaultserverport');
        defaultServerPort.setAttribute('value', serverPort);

        // add the new description
        descriptionTextNode = document.createTextNode(serverDescription);
        descriptionElement.appendChild(descriptionTextNode);
    }
    catch (e) {
        // The selected server is not in the list of known suggested servers.
        // Do not choose one by default and modify the user's
        // preferences, because the user may have hacked their preferences
        // through about:config to point to a server that is not in the
        // known servers list.

        // add the new description
        descriptionText = 'Please select a server';
        descriptionTextNode = document.createTextNode(descriptionText);
        descriptionElement.appendChild(descriptionTextNode);
    }
}

// handler for when the "Reset to Default" button is pressed
function dictResetServerChoicesToDefault() {

    // clear the existing settings
    try {
        var pref = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefBranch);
        pref.deleteBranch(PREFNAME_KNOWNSERVERS);
    }
    catch (e) {
        // ignore errors
        alert(e);
    }


    // add the default settings
    try {
        var i = 0;
        var dictserver;
        for (dictserver in DEFAULT_DICT_SERVERS) {

            dictSetKnownServerName(i, DEFAULT_DICT_SERVERS[dictserver].name);
            dictSetKnownServerPort(i, DEFAULT_DICT_SERVERS[dictserver].port);
            dictSetKnownServerDescription(i, DEFAULT_DICT_SERVERS[dictserver].description);

            i++;
        }
    }
    catch (e) {
        // ignore errors
        alert(e);
    }

    // repopulate the UI elements
    try {
        Startup();
    }
    catch (e) {
        // ignore errors
    }
}

// Best-effort to set the preferences to known good values if they cannot be read.
// This does not throw an exception.
function dictSetDefaultPreferences() {

    var serverName;
    var serverPort;

    try {
        serverName = dictGetDefaultServerName();
        dictSetDefaultServerName(serverName);
    }
    catch (e) {
        // ignore errors
    }

    try {
        serverPort = dictGetDefaultServerPort();
        dictSetDefaultServerPort(serverPort);
    }
    catch (e) {
        // ignore errors
    }

    try {
        // try to get info on the 0th server
        serverName = dictGetKnownServerName(0);
        serverPort = dictGetKnownServerPort(0);
    }
    catch (e) {
        // We could not get info on the 0th server.
        // The array must be missing/corrupt.
        // Make a best-effort to restore it
        dictResetServerChoicesToDefault();
    }
}



// This gets called automatically from Mozilla Application Suite
function Startup() {

    // set some default preferences, in case none exist
    dictSetDefaultPreferences();

    var listbox = document.getElementById("dictd-servers");

    // clear the contents of the listbox
    while (listbox.getRowCount() != 0) {
        listbox.removeItemAt(0);
    }

    var serverName;
    var serverPort;
    var i;

    for (i = 0;; i++) {

        try {
            serverName = dictGetKnownServerName(i);
            serverPort = dictGetKnownServerPort(i);

            // append this server to the list box
            var listitem = document.createElement('listitem');
            listitem.setAttribute('value', i);
            listitem.setAttribute('label', serverName);

            listbox.appendChild(listitem);
        }
        catch (e) {
            // we've reached the end of the array
            // (or encountered a malformed entry)
            break;
        }
    }

    // Try to find the index in the known servers that corresponds 
    // to the current default server so that we can select it.
    var defaultServerIndex = -1;

    var defaultServerName = dictGetDefaultServerName();
    var defaultServerPort = dictGetDefaultServerPort();

    for (i = 0;; i++) {

        try {
            serverName = dictGetKnownServerName(i);
            serverPort = dictGetKnownServerPort(i);

            if (serverName == defaultServerName &&
                serverPort == defaultServerPort) {
                defaultServerIndex = i;
                break;
            }
        }
        catch (e) {
            // we've reached the end of the array
            // (or encountered a malformed entry)
            break;
        }
    }

    listbox.selectedIndex = defaultServerIndex;
}



function dictPreferencesInit() {
    // Manually call the Startup since Mozilla Firefox doesn't call it
    Startup();
}


function dictPreferencesSave() {

    // Manually save prefs; Mozilla Firefox doesn't do it
    try {

        // set the server name
        var defaultServerName = document.getElementById("defaultservername");
        dictSetDefaultServerName(defaultServerName.value);

        // set the server port
        var defaultServerPort = document.getElementById("defaultserverport");
        dictSetDefaultServerPort(defaultServerPort.value);
    } 
    catch(e) {

        // HACK:
        // Switch out this alert because it causes problems on Firefox 0.8.
        // For some reason, setting the defaultserverport raises an exeception.
        // We don't really need to do this because all the dict servers 
        // that we offer listen on port 2628, which is the port that 
        // will be used if the defaultserverport option is corrupt.
        //
        // If we ever want to select a server that listens on a non-default
        // port, this will silently fail, and the source of the problem
        // could be hard to track down.
        if (0) {
            alert(
                'Unable to save preferences.\n' + 
                'If the problem persists, please file a bug report.');
        }
    }
}
